#!/usr/bin/python
#
# ftrace-test.py - automated test of ftrace functionality
#
# To Do for ftrace-test.py
#
# BUGS:
#
# OUTLINE:
#

MAJOR_VERSION = 0
MINOR_VERSION = 1
REVISION      = 0

import os, sys
import re, time, random
import commands
import ttc_test_utils

######################################
# Define some globals for this test suite

test_suite_name="FTRACE"
src_dir = "test-linux"
test_data_dir = "../test-data"

######################################
# Define some convenience classes and functions

def usage():
	print """Usage: %s [-h] <target>

where <target> is the name of the target to test.

-h	show this usage help
-V	show version information
""" % os.path.basename(sys.argv[0])

def test_setup(test_run):
	test_run.set_id("000", "ftrace-test setup")

	# test run prep
	print "Doing test preparation for %s tests..." % test_suite_name

	# make test data dir and build dir
	build_dir = "test-build/%s" % test_run.target
	rcode = test_run.do_command("install -d %s" % build_dir, 1)

	# set KBUILD_OUTPUT environment variable to build directory
	# Use a relative path from the kernel source directory, which
	#   will be one dir down from here.
	build_dir = "../test-build/%s" % test_run.target
	os.environ["KBUILD_OUTPUT"]=build_dir

	test_run.build_dir = build_dir

	# auto-detect that kernel is already present in src_dir
	# and skip the get_kernel automatically
	rcode = test_run.do_command('test -f %s/gk_marker' % src_dir)
	extra_data = ""
	if rcode:
		test_run.do_ttc("get_kernel -o %s" % src_dir, 1)
		test_run.do_command('echo "get_kernel completed" >%s/gk_marker' % src_dir)
	else:
		extra_data = "*** Skipping get_kernel (get_kernel marker found)"
		print extra_data

	# check for success
	# try to change to source directory
	try:
		os.chdir(src_dir)
	except:
		test_run.failure("Could not switch to kernel source directory after 'get_kernel'")
		sys.exit(1)

	# look for some 'normal' files at the root of the kernel source
	if not os.path.isfile('MAINTAINERS') or not os.path.isfile('Kbuild'):
		test_run.failure("Missing key files in kernel source directory after 'get_kernel'")
		sys.exit(1)
	
	# read Makefile, and get kernel version
	makelines = open("Makefile").readlines()
	version = 0
	minor = 0
	revision = 0
	extra = ""
	for line in makelines:
		if not version and line.startswith('VERSION'):
			version = line.split('=')[1].strip()
		if not minor and line.startswith('PATCHLEVEL'):
			minor = line.split('=')[1].strip()
		if not revision and line.startswith('SUBLEVEL'):
			revision = line.split('=')[1].strip()
		if not extra and line.startswith('EXTRAVERSION'):
			extra = line.split('=')[1].strip()
	ver_str = "%s.%s.%s%s" % (version, minor, revision, extra)
	test_run.result("Kernel version is %s" % ver_str)

	config_filename = os.path.join(test_run.build_dir, ".config")
	# first, remove old configuration, if present
	
	if os.path.isfile(config_filename):
		os.unlink(config_filename)
		if os.path.isfile(config_filename):
			test_run.failure("Could not remove pre-existing kernel config file from build dir.")
			sys.exit(1)

	# get default configuration
	rcode = test_run.do_ttc("get_config", 1)

def test_001(test_run):
	test_run.set_id("001", "build kernel with KFT options")
	print "Running test %s..." % test_run.id

	kernel_filename = os.path.join(test_run.build_dir, "vmlinux")

	# remove existing kernel file, if present
	if os.path.isfile(kernel_filename):
		os.unlink(kernel_filename)
		if os.path.isfile(kernel_filename):
			test_run.failure("Could not remove pre-existing kernel image file from build dir.")
			sys.exit(1)

	rcode = test_run.do_ttc("set_config CONFIG_KFT=y", 1)
	rcode = test_run.do_ttc("set_config CONFIG_KFT_STATIC_RUN=y", 1)
	#rcode = test_run.do_ttc("set_config CONFIG_KFT_SAVE_SP=n", 1)
	#rcode = test_run.do_ttc('set_config "CONFIG_CMDLINE+=\\" quiet\\""', 1)

	# do make oldconfig to resolve outstanding dependencies
	test_run.do_make_oldconfig()

	if not test_run.check_config("CONFIG_KFT","y"):
		test_run.failure("Could not set CONFIG_KFT=y in kernel configuration")
		sys.exit(1)

	if not test_run.check_config("CONFIG_KFT_STATIC_RUN","y"):
		test_run.failure("Could not set CONFIG_KFT_STATIC_RUN=y in kernel configuration")
		# don't exit, can still try some tests

	# set ftrace configuration
	fd = open("kernel/kftstatic.conf","w")
	fd.write(
"""begin
   trigger start entry start_kernel
   trigger stop entry to_userspace
   filter mintime 500
end""")
	fd.close()

	test_run.set_localversion()

	# build kernel
	rcode = test_run.do_ttc("kbuild")
	if rcode:
		test_run.failure("Could not build kernel")

	# there should be a vmlinux at root of build directory.
	if not os.path.isfile(kernel_filename):
		test_run.failure("No 'vmlinux' found after build")
	else:
		test_run.success("Built kernel successfully with KFT options - found 'vmlinux' after build")

	# install kernel
	rcode = test_run.do_ttc("kinstall", 1)
	if rcode:
		test_run.failure("Could not install kernel")

	# reset the board - use strongest reboot possible
	test_run.reset_target("reset")
	if not test_run.check_localversion():
		test_run.reset_target("reboot")
		if not test_run.check_localversion():
			test_run.failure("Could not verify kernel running on target board")


	if test_run.check_localversion():
		test_run.success("Kernel %s installed and booted on board" % test_run.kernel_id)
	else:
		test_run.failure("Could not verify kernel running on target board")

	# check if appropriate files are available
	rcode, result = test_run.run_on_target("cat /proc/kft") 
	if rcode:
		test_run.failure("Could not cat /proc/kft")
	else:
		test_run.success("Found /proc/kft on target.", result)

	rcode, result = test_run.run_on_target("cat /proc/kft_data >/tmp/boot.log") 
	if rcode:
		test_run.failure("Could not cat /proc/kft_data")

	# copy boot.log to host
	tmp_filename=test_data_dir + "/" + "boot.log.%d" % random.randint(1,100000)
	rcode, result = test_run.do_ttc("cp target:/tmp/boot.log %s" % tmp_filename) 
	test_run.tmp_kftlogname = tmp_filename

	# FIXTHIS - check for basic boot.log goodness here
	lines = open(tmp_filename).readlines()
	good_trace_header_found = 0
	for line in lines:
		if line.startswith("Kernel Function Trace Run ID"):
			good_trace_header = 1

	if good_trace_header_found:
		test_run.success("Found good trace header in boot.log")	
	else:
		test_run.failure("Could not find good trace header in boot.log")

#############################################
# KFT-002 - test addr2sym
#
def test_002(test_run):
	test_run.set_id("002", "test addr2sym")
	print "Running test %s..." % test_run.id

	# FIXTHIS execute addr2sym
	# FIXTHIS look for names in place of symbols

#############################################
# KFT-003 - test kd
#
def test_003(test_run):
	test_run.set_id("003", "test kd")
	print "Running test %s..." % test_run.id

	# FIXTHIS run kd on boot.log
	# FIXTHIS look function names, sorted by time

#############################################
# KFT-004 - test dynamic trace
#
def test_004(test_run):
	test_run.set_id("004", "test dynamic trace")
	print "Running test %s..." % test_run.id

	# FIXTHIS - testplan says to reconfigure, recompile and redeploy
	# kernel with STATIC_TRACE=n.  Right now I don't care.

	# FIXTHIS create trace config (do_fork)
	temp_conf = test_data_dir+"/do_fork.conf"
	fd = open(temp_conf,"w")
	fd.write("""
new
begin
  trigger entry do_fork
  trigger exit do_fork
  filter mintime 50
end
""")
	fd.close()

	# copy System.map 
	test_run.do_command("cp %s/System.map %s" % (build_dir, test_data_dir))

	# lookup syms in trace config
	test_run.do_command("scripts/sym2addr <%s %s/System.map >%s.mapped" % (temp_conf, test_data_dir, temp_conf))

	# set trace config (cat /tmp/trace.conf.mapped >/proc/kft
	test_run.do_ttc("cp %s.mapped target:/tmp/do_fork.conf.mapped" % temp_conf)
	test_run.run_on_target("cat do_fork.conf.mapped >/proc/kft")

	# verify that trace config set correctly
	# prime trace
	# collect results
	# verify results are OK

def test_cleanup(test_run):
	#os.unlink(test_run.tmp_kftlogname)
	pass
	
def main():
	# get list of valid targets
	(rcode, result) = commands.getstatusoutput('ttc list -q')
	tlist = result.split("\n");
	if rcode:
		print "Error: Problem running 'ttc list'"
		sys.exit(1)

	# Parse the command line
	if len(sys.argv)<2:
		print "Error: missing target to run test on."
		usage()
		print "Available targets are:"
		for t in tlist:
			print "   %s" % t
		print
		sys.exit(1)

	if '-h' in sys.argv:
		usage()
		sys.exit(1)

	if '-V' in sys.argv:
		print "target-test.py Ver. %d.%d.%d" % (MAJOR_VERSION, MINOR_VERSION, REVISION)
		sys.exit(1)

	target = sys.argv[1]

	# make sure there are no conflicts - check for existing
	# TC_TARGET environment variable
	if os.environ.has_key("TC_TARGET"):
		print "Error: found TC_TARGET in environment."
		print "Please do not run this test inside an existing ttc target environment."
		print "Aborting test."
		sys.exit(1)

	test_run = ttc_test_utils.test_run_class(test_suite_name, target)

	# make directory for test output
	# remove leading ../ from test_data_dir, since we haven't changed
	# to the kernel source dir yet
	test_run.do_command("install -d %s" % test_data_dir[3:], 1)

	test_run.start_log(test_data_dir[3:])

	if target not in tlist:
		test_run.failure("Error: target '%s' not found in ttc configuration" % target)
		print "\n###########################################"
		print "Results summary:"
		test_run.show_summary_results()
		sys.exit(1)

	# set up test directories
	test_setup(test_run)

	# run remainder of tests
	test_001(test_run) # set KFT options, build, boot, retrieve boot.log
	test_002(test_run) # test addr2sym
	test_003(test_run) # test kd (basic usage)
	test_004(test_run) # test dynamic trace
	#test_005(test_run) # test entry trigger
	#test_006(test_run) # test exit trigger
	#test_007(test_run) # test mintime filter
	#test_008(test_run) # test noints filter
	#test_009(test_run) # test kd command-line options
	#test_010(test_run) # test timing accuracy

	test_cleanup(test_run)
	test_run.close_log()

	print "\n###########################################"
	print "Results summary:"
	test_run.show_summary_results()

if __name__=="__main__":
	main()
