#!/usr/bin/python
#
# kft-test.py - automated test of KFT functionality
#
# To Do for kft-test.py
# * make sure that no existing env. vars are set to conflict with our target
#
# BUGS:
#
# OUTLINE:
# items to test:
# configuration:
# 	ability to set KFT options (001)
# build:
# 	ability to compile kernel with static KFT set (001)
# run:
#	ability to boot kernel with static KFT set (001)
# static trace:
#	static trace captured data (001)
#	data was valid (001)
#	   FIXTHIS - how is 'valid' defined?? (see check_log())
# dynamic trace:
#	load a trace config (004)
# trace features:
#   triggers
#	test entry trigger
#	test exit trigger
#	test time trigger
#   filters
#	test mintime filter (005)
#	test noint filter
#	test onlyint filter
#   function list:
#	test function list filter
# miscellaneous
#   manual test start
#   manual test stop
# trace processing tools:
# 	use addr2sym to convert addresses to symbols (002)
#	use sym2addr to convert symbols to addresses (004)
#	test kd (basic usage) (002)??
#	test kd command line options
# accuracy:
#	test timing accuracy
#	check for lots of "not found" entries
#	run trace a long time, and see if time gets worse??
#	check for trace events out of order
# performance:
#	test boot time w/wo tracer enabled
#	test dynamic trace time w/wo tracer enabled
#
# todo:
# write routine to parse KFT log (especially header)

MAJOR_VERSION = 1
MINOR_VERSION = 0
REVISION      = 0

import os, sys
import re, time, random
import commands
import ttc_test_utils

available_tests = {
	"1":"build kernel with KFT options, install and check /proc",
	"2":"test addr2sym",
	"3":"test kd",
	"4":"test dynamic trace (including entry, exit, mintime=1)",
	"5":"test time trigger",
	"7":"test time filters",
	"8":"test noints filter",
	"9":"test kd command-line options",
	"10":"test timing accuracy",
}

######################################
# Define some globals for this test suite

test_suite_name="KFT"
src_dir = "test-linux"
test_data_dir = "../test-data"

######################################
# Define some convenience classes and functions

def usage():
	prog_name = os.path.basename(sys.argv[0])
	print """Usage: %s [-h] [-V] [-t <tests>] <target>

where <target> is the name of the target to test.

%s is a program to test KFT functionality in a kernel
(using ttc commands).  Test results are written to
the test-data directory.

-t <tests>  Run only the specified tests. Tests are specified
            with a comma-separated list.  
-l          List available tests
-h	    Show this usage help
-V	    Show version information
""" % (prog_name, prog_name)

test_run_g = None

def print_log(message):
	global test_run_g

	test_run_g.print_and_log_message(message)

def test_setup(test_run):
	test_run.set_id("000", "kft-test setup")

	# test run prep
	print_log("Doing test preparation for %s tests..." % test_suite_name)

	# make test data dir and build dir
	build_dir = "test-build/%s" % test_run.target
	rcode = test_run.do_command("install -d %s" % build_dir, 1)

	# set KBUILD_OUTPUT environment variable to build directory
	# Use a relative path from the kernel source directory, which
	#   will be one dir down from here.
	build_dir = "../test-build/%s" % test_run.target
	os.environ["KBUILD_OUTPUT"]=build_dir

	test_run.build_dir = build_dir

	# auto-detect that kernel is already present in src_dir
	# and skip the get_kernel automatically
	rcode = test_run.do_command('test -f %s/gk_marker' % src_dir)
	extra_data = ""
	if rcode:
		test_run.do_ttc("get_kernel -o %s" % src_dir, 1)
		test_run.do_command('echo "get_kernel completed" >%s/gk_marker' % src_dir)
	else:
		extra_data = "*** Skipping get_kernel (get_kernel marker found)"
		print_log(extra_data)

	# check for success
	# try to change to source directory
	try:
		os.chdir(src_dir)
	except:
		test_run.failure("Could not switch to kernel source directory after 'get_kernel'")
		sys.exit(1)

	# look for some 'normal' files at the root of the kernel source
	if not os.path.isfile('MAINTAINERS') or not os.path.isfile('Kbuild'):
		test_run.failure("Missing key files in kernel source directory after 'get_kernel'")
		sys.exit(1)
	
	# read Makefile, and get kernel version
	makelines = open("Makefile").readlines()
	version = 0
	minor = 0
	revision = 0
	extra = ""
	for line in makelines:
		if not version and line.startswith('VERSION'):
			version = line.split('=')[1].strip()
		if not minor and line.startswith('PATCHLEVEL'):
			minor = line.split('=')[1].strip()
		if not revision and line.startswith('SUBLEVEL'):
			revision = line.split('=')[1].strip()
		if not extra and line.startswith('EXTRAVERSION'):
			extra = line.split('=')[1].strip()
	ver_str = "%s.%s.%s%s" % (version, minor, revision, extra)
	test_run.result("Kernel version is %s" % ver_str)

	config_filename = os.path.join(test_run.build_dir, ".config")
	# first, remove old configuration, if present
	
	if os.path.isfile(config_filename):
		os.unlink(config_filename)
		if os.path.isfile(config_filename):
			test_run.failure("Could not remove pre-existing kernel config file from build dir.")
			sys.exit(1)

	# get default configuration
	rcode = test_run.do_ttc("get_config", 1)

def test_001(test_run):
	test_run.set_id("001", "build kernel with KFT options, install and check /proc")
	print_log("Running test %s..." % test_run.id)

	kernel_filename = os.path.join(test_run.build_dir, "vmlinux")

	# remove existing kernel file, if present
	if os.path.isfile(kernel_filename):
		os.unlink(kernel_filename)
		if os.path.isfile(kernel_filename):
			test_run.failure("Could not remove pre-existing kernel image file from build dir.")
			sys.exit(1)

	rcode = test_run.do_ttc("set_config CONFIG_KFT=y", 1)
	rcode = test_run.do_ttc("set_config CONFIG_KFT_STATIC_RUN=y", 1)
	rcode = test_run.do_ttc("set_config CONFIG_KFT_SAVE_SP=n", 1)
	#rcode = test_run.do_ttc('set_config "CONFIG_CMDLINE+=\\" quiet\\""', 1)

	# do make oldconfig to resolve outstanding dependencies
	test_run.do_make_oldconfig()

	if not test_run.check_config("CONFIG_KFT","y"):
		test_run.failure("Could not set CONFIG_KFT=y in kernel configuration")
		sys.exit(1)

	if not test_run.check_config("CONFIG_KFT_STATIC_RUN","y"):
		test_run.failure("Could not set CONFIG_KFT_STATIC_RUN=y in kernel configuration")
		# don't exit, can still try some tests

	# set kftstatic.conf
	fd = open("kernel/kftstatic.conf","w")
	fd.write(
"""begin
   trigger start entry start_kernel
   trigger stop entry to_userspace
   filter mintime 500
end""")
	fd.close()

	test_run.set_localversion()

	# build kernel
	rcode = test_run.do_ttc("kbuild")
	if rcode:
		test_run.failure("Could not build kernel")

	# there should be a vmlinux at root of build directory.
	if not os.path.isfile(kernel_filename):
		test_run.failure("No 'vmlinux' found after build")
	else:
		test_run.success("Built kernel successfully with KFT options - found 'vmlinux' after build")

	# install kernel
	rcode = test_run.do_ttc("kinstall", 1)
	if rcode:
		test_run.failure("Could not install kernel")

	# reset the board - use strongest reboot possible
	test_run.reset_target("reset")
	if not test_run.check_localversion():
		test_run.reset_target("reboot")
		if not test_run.check_localversion():
			test_run.failure("Could not verify kernel running on target board")


	if test_run.check_localversion():
		test_run.success("Kernel %s installed and booted on board" % test_run.kernel_id)
	else:
		test_run.failure("Could not verify kernel running on target board")

	# check if appropriate files are available
	rcode, result = test_run.run_on_target("cat /proc/kft") 
	if rcode:
		test_run.failure("Could not cat /proc/kft")
	else:
		test_run.success("Found /proc/kft on target.", result)

	rcode, result = test_run.run_on_target("cat /proc/kft_data >/tmp/boot.log") 
	if rcode:
		test_run.failure("Could not cat /proc/kft_data")
	else:
		test_run.success("Did 'cat /proc/kft_data' on target")

###########################################
# check_log_file - a routine to check various attributes about a log file
# this routine checks for:
# 	1) title which matches expected trace_run_id
# 	2) line starting with Logging started
# 	3) line starting with Logging stopped
# 	4) line starting with Execution time filter count
# 	5) number of entries exceeds min_expected_entries
# 	6) number of entries exceeds min_expected_entries
#	7) lines 3, 6, and 10 were found in the sample space
#	8) strings where function addresses are expected start with "0x"
#	9) entry times for function addresses were increasing in the samples
#
def check_log_file(test_run, min_expected_entries, extra_checks=[]):
	filename = test_run.log_filename
	lines = open(filename).readlines()

	line_checks = [
		"Kernel Function Trace Run ID %s" % test_run.trace_run_id,
		"Logging started",
		"Logging stopped",
		]

	line_checks.extend(extra_checks)

	line_results = {}
	for test in line_checks:
		line_results[test] = 0

	entry_pat = re.compile("Number of entries after filters = ([0-9]+)")
	enough_entries_found = 0
	for line in lines:
		for test in line_checks:
			if line.startswith(test):
				line_results[test] = 1
		m = entry_pat.match(line)
		if m:
			entry_count = int(m.group(1))
			if entry_count > min_expected_entries:
				enough_entries_found = 1

	expected_elements = len(line_checks) + 1
	good_elements_found = 0
	for test in line_checks:
		if line_results[test]:
			good_elements_found += 1
		else:
			test_run.result("Missing '%s'" % test)

	if enough_entries_found:
		good_elements_found += 1
	else:
		test_run.failure("Expected more than %d entries, but only saw %d" % (min_expected_entries, entry_count))

	if good_elements_found == expected_elements:
	 	test_run.success("trace %s has %d of %d expected elements" % (filename, good_elements_found, expected_elements))
	else:
	 	test_run.failure("trace %s only has %d of %d expected elements" % (filename, good_elements_found, expected_elements))

	# examine a sampling of lines, and check for addresses
	sample_locs = [3,6,10]
	samples = []

	in_funcs = 0
	i = 0
	for line in lines:
		if not in_funcs: 
			if line.startswith("-------"):
				in_funcs = 1
			continue
		i += 1
		if i in sample_locs:
			tuple = line.split()
			# save entry, delta, pid, func_addr
			samples.append((i, tuple[0], tuple[1], tuple[2], tuple[3]))

	print_log("samples=%s" % samples)

	# verify some stuff about the samples
	# are addresses in 0x... format?
	data_ok = 1
	if not samples:
		data_ok = 0
		test_run.failure("Could not find entry samples in log file %s" % filename, "samples=%s" % samples)

	# do function names start with '0x'?
	if data_ok and (not samples[0][4].startswith("0x") or \
		not samples[1][4].startswith("0x") or \
		not samples[2][4].startswith("0x") ):
		test_run.failure("Data in log file %s doesn't have valid addresses for one of entries %s" %  (filename, sample_locs), "samples=%s" % samples)
		data_ok = 0

	# is entry increasing?
	if samples and int(samples[0][1]) > int(samples[1][1]) or \
		int(samples[1][1]) > int(samples[2][1]):
		test_run.failure("Data in log file %s doesn't have increasing entry times" %  filename, "samples=%s" % samples)
		data_ok = 0

	return data_ok

################
# gen_listfile
def gen_listfile(test_run, trace_name):
	infile = test_run.log_filename
	outfile = test_data_dir + "/%s.lst.%s" % (trace_name, test_run.run_uid)
	test_run.do_command("scripts/addr2sym <%s -m %s/System.map >%s" % (infile, test_data_dir, outfile))
	test_run.list_filename = outfile
	return outfile

def check_file_for_pat(filename, pat):
	patc = re.compile(pat)
	found_line = 0
	for line in open(filename).readlines():
		if patc.match(line):
			found_line = 1
	return found_line

def check_log_for_pat(test_run, pat):
	return check_file_for_pat(test_run.log_filename, pat)

def get_line_match_from_file(filename, pat):
	patc = re.compile(pat)
	for line in open(filename).readlines():
		if patc.match(line):
			return line
	return ""

def get_line_match_from_log(test_run, pat):
	return get_line_match_from_file(test_run.log_filename, pat)

def get_line_match_from_lst(test_run, pat):
	return get_line_match_from_file(test_run.list_filename, pat)

def test_001a(test_run):
	test_run.set_id("001a", "test boot.log data")
	print_log("Running test %s..." % test_run.id)

	log_ok = collect(test_run, "boot")
	if not log_ok:
		return

	# check for basic boot.log goodness
	test_run.trace_run_id = "0"
	log_ok = check_log_file(test_run, 1000,
		["Execution time filter count"])
	if log_ok:
		test_run.success("Data in log file %s looks OK" % test_run.log_filename)

#############################################
# KFT-002 - test addr2sym
#
def test_002(test_run):
	test_run.set_id("002", "test addr2sym")
	print_log("Running test %s..." % test_run.id)

	test_run.do_command("cp %s/System.map %s" % (test_run.build_dir, test_data_dir))

	# convert to "boot.lst"
	gen_listfile(test_run, "boot")

	# save off a few sample lines (which should have addresses)
	infile = test_run.log_filename
	lines = open(infile).readlines()
	sample_locs = [3,6,10]
	addr_samples = []

	in_funcs = 0
	i = 0
	for line in lines:
		if not in_funcs: 
			if line.startswith("-------"):
				in_funcs = 1
			continue
		i += 1
		if i in sample_locs:
			tuple = line.split()
			func_addr = tuple[3]
			called_at_addr = tuple[4]
			addr_samples.append((i, func_addr, called_at_addr))

	test_run.result("Address samples=%s" % addr_samples)

	# look for symbols (names) in place of addresses
	outfile = test_run.list_filename
	lines = open(outfile).readlines()
	name_samples = []

	in_funcs = 0
	i = 0
	for line in lines:
		if not in_funcs: 
			if line.startswith("-------"):
				in_funcs = 1
			continue
		i += 1
		if i in sample_locs:
			tuple = line.split()
			func_addr = tuple[3]
			called_at_addr = tuple[4]
			name_samples.append((i, func_addr, called_at_addr))

	test_run.result("Name samples=%s" % name_samples)

	if not( addr_samples[0][0] == name_samples[0][0] and \
		addr_samples[1][0] == name_samples[1][0] and \
		addr_samples[2][0] == name_samples[2][0] ):
		test_run.failure("Didn't collect similar samples from %s and %s" % (infile, outfile))
		return

	# test whether addresses have changed to name
	print_log("Here are some matches #####################")
	print_log( str(addr_samples[0][1] != name_samples[0][1]))
	print_log( str(re.match("[a-z_]+", name_samples[0][1])))
	print_log( str(addr_samples[0][2] != name_samples[0][2]))

	print_log( str(addr_samples[1][1] != name_samples[1][1]))
	print_log( str(re.match("[a-z_]+", name_samples[1][1])))
	print_log( str(addr_samples[1][2] != name_samples[1][2]))

	print_log( str(addr_samples[2][1] != name_samples[2][1]))
	print_log( str(re.match("[a-z_]+", name_samples[2][1])))
	print_log( str(addr_samples[2][2] != name_samples[2][2]))

	if addr_samples[0][1] != name_samples[0][1] and \
		re.match("[a-z_]+", name_samples[0][1]) and \
		addr_samples[0][2] != name_samples[0][2] and \
		addr_samples[1][1] != name_samples[1][1] and \
		re.match("[a-z_]+", name_samples[1][1]) and \
		addr_samples[1][2] != name_samples[1][2] and \
		addr_samples[2][1] != name_samples[2][1] and \
		re.match("[a-z_]+", name_samples[2][1]) and \
		addr_samples[2][2] != name_samples[2][2]:
		test_run.success("Addresses converted to names in file %s" % outfile)
	else:
		test_run.failure("Addresses NOT converted to names in file %s" % outfile)

#############################################
# KFT-003 - test kd
#
def test_003(test_run):
	test_run.set_id("003", "test kd")
	print_log("Running test %s..." % test_run.id)

	# run kd on boot.lst
	infile = test_run.list_filename
	rcode, result = test_run.do_command_result("scripts/kd %s" % infile)

	if rcode:
		test_run.failure("Error %s running kd on %s" % (rcode, infile))
		return

	# look for function names, sorted by time
	lines = result.split('\n')
	sample_locs = [3,6,10]

	samples = []

	i = 0
	for line in lines:
		i += 1
		if i in sample_locs:
			# line should have "function count time average local"
			tuple = line.split()
			# save function, time
			samples.append((i, tuple[0], tuple[2]))

	print_log("samples=%s" % samples)

	# check for alpha names in first column
	data_ok = 1
	if not( re.match("[a-z_]+", samples[0][1]) and \
		re.match("[a-z_]+", samples[1][1]) and \
		re.match("[a-z_]+", samples[2][1]) ):
		test_run.failure("function names not found in Function column")
		data_ok = 0

	# check for decreasing times
	if not( int(samples[0][2]) > int(samples[1][2]) and \
		int(samples[1][2]) > int(samples[2][2]) ):
		test_run.failure("function durations not decreasing in Time column")
		data_ok = 0

	if data_ok:
		test_run.success("KD output data is OK")

###############################
# set_trace_conf
# this converts symbols to addrs, then writes the trace config to /proc/kft
# it verifies that the trace's run id matches what is expected
# the trace_run_id is saved in test_run for later use
def set_trace_conf(test_run, name, conf):
	# get current trace run ID
	try:
		rcode, result = test_run.run_on_target("cat /proc/kft")
		line = result.split('\n')[0]
		m = re.match("status: run id ([0-9]+)", line)
		run_id = m.group(1)
	except:
		test_run.failure("Could not get current trace run id on target")
		run_id = -1

	# write trace config to a file
	conf_name = name + ".conf"
	temp_conf = test_data_dir+"/" + conf_name
	fd = open(temp_conf,"w")
	fd.write(conf)
	fd.close()

	# copy System.map 
	test_run.do_command("cp %s/System.map %s" % (test_run.build_dir, test_data_dir))

	# lookup syms in trace config
	test_run.do_command("scripts/sym2addr %s %s/System.map >%s.mapped" % (temp_conf, test_data_dir, conf_name))

	# set trace config
	test_run.do_ttc("cp %s.mapped target:/tmp/%s.mapped" % (conf_name, conf_name))
	rcode, result = test_run.run_on_target("cat /tmp/%s.mapped >/proc/kft" % conf_name)

	# verify that trace config set correctly
	set_conf_ok = 1
	rcode, result = test_run.run_on_target("dmesg | tail -n 4")
	result = result[:-1]	# chop trailing '\n'

	print_log("result=%s" % result)
	if not ("KFT: new kft run installed" in result.split('\n')[-1]):
		test_run.failure("Did not set %s trace config properly" % name, "result from cat /tmp/%s.mapped >/proc/kft=%s" % (conf_name, result))
		set_conf_ok = 0

	# get current trace run ID
	try:
		rcode, result = test_run.run_on_target("cat /proc/kft")
		lines = result.split('\n')
		m = re.match("status: run id ([0-9]+)", lines[0])
		new_run_id = m.group(1)
	except:
		test_run.failure("Could not get current trace run id on target")

	# check for id = last_id + 1
	set_conf_ok = 1
	test_run.trace_conf_lines = lines
	if int(new_run_id) == int(run_id)+1:
		test_run.trace_run_id = new_run_id
	else:
		test_run.trace_run_id = "-1"
		test_run.failure("Unexpected run_id for %s trace: run_id=%s, new_run_id=%s" % (name, run_id, new_run_id))
		set_conf_ok = 0

	return set_conf_ok

def check_conf(test_run, index, expected_string):
	conf_ok = 0
	i = 0
	for test_line in test_run.trace_conf_lines:
		i += 1
		test_line = test_line.strip()
		if i==index:
			print_log("conf_line[%d]='%s'" % (index, test_line))
			saved_line = test_line
		if test_line == expected_string:
			conf_ok = 1

	if not conf_ok:
		test_run.failure("conf not set properly for trace (line='%s')" % saved_line)

	return conf_ok

# collect - retrieve a trace from the target, and put it in the test data
# directory
# returns 1 on success.
# on failure, the failure is logged
def collect(test_run, trace_name):
	# collect results
	cmd = "cat /proc/kft_data >/tmp/%s.log" % trace_name
	rcode, result = test_run.run_on_target(cmd) 
	if rcode:
		test_run.failure("Could not cat /proc/kft_data")
		return 0

	# generate a unique logname (based on this test run)
	ruid = test_run.run_uid
	tmp_filename=test_data_dir + "/%s.log.%s" % (trace_name, ruid)

	cmd = "cp target:/tmp/%s.log %s" % (trace_name, tmp_filename)
	rcode = test_run.do_ttc(cmd)

	# save name for later use
	test_run.log_filename = tmp_filename

	if rcode:
		test_run.failure("Could not get log for %s trace" % trace_name)
		return 0

	# generate .lst file
	gen_listfile(test_run, trace_name)

	return 1


#############################################
# KFT-004 - test dynamic trace (including entry, exit, mintime=1)
# this also tests trigger start on function entry
# this also tests trigger stop on function exit
# this also tests filter mintime of 1
#
def test_004(test_run):
	test_run.set_id("004", "test dynamic trace (including entry, exit, mintime=1)")
	print_log("Running test %s..." % test_run.id)

	# FIXTHIS - testplan says to reconfigure, recompile and redeploy
	# kernel with STATIC_TRACE=n.  Right now I don't care.

	trace_name = "do_fork"

	set_conf_ok = set_trace_conf(test_run, trace_name,
"""
new
begin
  trigger start entry do_fork
  trigger stop exit do_fork
end
""")

	if set_conf_ok:
		set_conf_ok = check_conf(test_run, 6, "filter mintime 0")
	if not set_conf_ok:
		return

	test_run.success("Correctly registered configuration for trace", "result from /proc/kft=%s" % '\n'.join(test_run.trace_conf_lines))

	# prime trace
	test_run.run_on_target("echo prime >/proc/kft")

	# execute an arbitrary command on the target, to activate the trace
	test_run.run_on_target("ls")

	# collect results
	collect_ok = collect(test_run, trace_name)
	if not collect_ok:
		return

	# check for general goodness
	log_ok = check_log_file(test_run, 500)
	if log_ok:
		test_run.success("Data in log file %s looks OK" % test_run.log_filename)

#############################################
# KFT-005 - test time trigger
#
def test_005(test_run):
	test_run.set_id("005", "test time trigger")
	print_log("Running test %s..." % test_run.id)

	trace_name = "time_trigger"

	set_conf_ok = set_trace_conf(test_run, trace_name,
"""
new
begin
  trigger start entry do_fork
  trigger stop time 50000
  filter mintime 100
end
""")

	# check for trace conf correctness
	if set_conf_ok:
		set_conf_ok = check_conf(test_run, 5, "trigger stop at time 50000")
	if set_conf_ok:
		set_conf_ok = check_conf(test_run, 6, "filter mintime 100")
	if not set_conf_ok:
		return

	test_run.run_on_target("echo prime >/proc/kft")
	test_run.run_on_target("ls ; sleep 1")
	collect_ok = collect(test_run, trace_name)
	if not collect_ok:
		return

	log_ok = check_log_file(test_run, 500,
		["Execution time filter count"])
	if not log_ok:
		return

	# check the log for interesting stuff
	# check for trigger stop reason
	pat = "Logging stopped.*by time at 50000 usec"
	if not check_log_for_pat(test_run, pat):
		test_run.failure("Didn't find trigger stop marker in log file")
		return

	# read last line of log, and see if entry time is close to 50000
	for line in open(test_run.log_filename).readlines():
		pass
	print_log("last line=%s" % line)

	entry_time = int(line.split()[0])
	if entry_time > 49800 and entry_time < 50200:
		test_run.success("Trace triggered close to 50000 usecs")
	else:
		test_run.failure("Trace did not trigger close to 50000 usces")


#############################################
# KFT-007 - test time filters
# make sure number of entries after with 'filter mintime 10' exceeds the
# number of entries with 'filter mintime 50'
#
def test_007(test_run):
	test_run.set_id("007", "test time filters")
	print_log("Running test %s..." % test_run.id)

	trace_name = "mintime-5"

	set_conf_ok = set_trace_conf(test_run, trace_name,
"""
new
begin
  trigger start entry do_fork
  trigger stop exit do_fork
  filter noints
  filter mintime 5
end
""")

	if set_conf_ok:
		set_conf_ok = check_conf(test_run, 6, "filter mintime 5")
	if set_conf_ok:
		set_conf_ok = check_conf(test_run, 6, "filter noints")
	if not set_conf_ok:
		return

	test_run.run_on_target("echo prime >/proc/kft")
	test_run.run_on_target("ls")
	collect_ok = collect(test_run, trace_name)
	if not collect_ok:
		return

	log_ok = check_log_file(test_run, 100,
		["Execution time filter count",
		"No Interrupt functions filter count"])
	
	if not log_ok:
		return

	# get number of entries and number filtered
	pat = "Number of entries after filters = ([0-9]+)"
	line = get_line_match_from_log(test_run, pat)
	m = re.match(pat, line)
	entry_count1 = int(m.group(1))

	pat = "Execution time filter count = ([0-9]+)"
	line = get_line_match_from_log(test_run, pat)
	m = re.match(pat, line)
	filter_count1 = int(m.group(1))

	test_run.result("Number of entries filtered with 'filter mintime 5' is %d" % filter_count1)

	trace_name = "mintime-50"

	set_conf_ok = set_trace_conf(test_run, trace_name,
"""
new
begin
  trigger start entry do_fork
  trigger stop exit do_fork
  filter noints
  filter mintime 50
end
""")

	# check for trace conf correctness
	if set_conf_ok:
		set_conf_ok = check_conf(test_run, 6, "filter mintime 50")
	if set_conf_ok:
		set_conf_ok = check_conf(test_run, 6, "filter noints")
	if not set_conf_ok:
		return

	test_run.run_on_target("echo prime >/proc/kft")
	test_run.run_on_target("ls")
	collect_ok = collect(test_run, trace_name)
	if not collect_ok:
		return

	log_ok = check_log_file(test_run, 20,
		["Execution time filter count",
		"No Interrupt functions filter count"])
	if not log_ok:
		return

	# get number of entries and number filtered
	pat = "Number of entries after filters = ([0-9]+)"
	line = get_line_match_from_log(test_run, pat)
	m = re.match(pat, line)
	entry_count2 = int(m.group(1))

	pat = "Execution time filter count = ([0-9]+)"
	line = get_line_match_from_log(test_run, pat)
	m = re.match(pat, line)
	filter_count2 = int(m.group(1))

	test_run.result("Number of entries filtered with 'filter mintime 50' is %d" % filter_count2)

	if filter_count2 > filter_count1:
		test_run.success("'filter mintime 50' filtered more events than 'filter mintime 5'")
	else:
		test_run.failure("'filter mintime 50' did NOT filter more events than 'filter mintime 5'")


#############################################
# KFT-008 - test noints filter
# run a test with ints and with noints, and see if noints has interrupt entries
#
def test_008(test_run):
	test_run.set_id("008", "test noints filter")
	print_log("Running test %s..." % test_run.id)

	trace_name = "with-ints"

	set_conf_ok = set_trace_conf(test_run, trace_name,
"""
new
begin
  trigger start entry do_fork
  trigger stop exit do_fork
  filter mintime 0
end
""")

	if set_conf_ok:
		set_conf_ok = check_conf(test_run, 6, "filter mintime 0")
	if not set_conf_ok:
		return

	test_run.run_on_target("echo prime >/proc/kft")
	test_run.run_on_target("ls")
	collect_ok = collect(test_run, trace_name)
	if not collect_ok:
		return

	log_ok = check_log_file(test_run, 100)
	if not log_ok:
		return

	# get number of entries and number filtered
	pat = "Number of entries after filters = ([0-9]+)"
	line = get_line_match_from_log(test_run, pat)
	m = re.match(pat, line)
	entry_count1 = int(m.group(1))

	pat = "Total entries filtered = ([0-9]+)"
	line = get_line_match_from_log(test_run, pat)
	m = re.match(pat, line)
	filter_count1 = int(m.group(1))

	test_run.result("Number of entries filtered without 'noints' is %d" % filter_count1)

	trace_name = "noints"

	set_conf_ok = set_trace_conf(test_run, trace_name,
"""
new
begin
  trigger start entry do_fork
  trigger stop exit do_fork
  filter noints
  filter mintime 0
end
""")

	# check for trace conf correctness
	if set_conf_ok:
		set_conf_ok = check_conf(test_run, 6, "filter noints")
	if not set_conf_ok:
		return

	test_run.run_on_target("echo prime >/proc/kft")
	test_run.run_on_target("ls")
	collect_ok = collect(test_run, trace_name)
	if not collect_ok:
		return

	log_ok = check_log_file(test_run, 20,
		["No Interrupt functions filter count"])
	if not log_ok:
		return

	# get number of entries and number filtered
	pat = "Number of entries after filters = ([0-9]+)"
	line = get_line_match_from_log(test_run, pat)
	m = re.match(pat, line)
	entry_count2 = int(m.group(1))

	pat = "Total entries filtered = ([0-9]+)"
	line = get_line_match_from_log(test_run, pat)
	m = re.match(pat, line)
	filter_count2 = int(m.group(1))

	test_run.result("Number of entries filtered with 'filter noints' is %d" % filter_count2)

	if filter_count2 > filter_count1:
		test_run.success("'filter noints' filtered more events than without the filter")
	else:
		test_run.failure("'filter noints' did NOT filter more events than without the filter")

#############################################
# KFT-009 - test kd command-line options
#
# this code assumes that boot.lst exists (that test 001 and 001a was run)
#
def test_009(test_run):
	test_run.set_id("009", "test kd command-line options")
	print_log("Running test %s..." % test_run.id)

	options_ok = 1

	# check for usage line
	rcode, result = test_run.do_command_result("scripts/kd -h")
	if not result.split()[0].startswith("usage:"):
		options_ok = 0
		test_run.failure("Missing usage for scripts/kd (-h not supported?)")

	# use boot.lst for tests
	infile = test_data_dir + "/boot.lst.%s" % test_run.run_uid

	# can use a hardcoded filename for testing kft-test.py
	#infile = test_data_dir + "/boot.lst.1152"

	# collect baseline results
	rcode, base_results = test_run.do_command_result("scripts/kd %s" % infile)
	if rcode:
		test_run.failure("Error %s running kd on %s" % (rcode, infile))
		return
	base_lines = base_results.split('\n')
	
	# try a long listing
	rcode, long_results = test_run.do_command_result("scripts/kd -l %s" % infile)
	if rcode:
		test_run.failure("Error %s running kd on %s" % (rcode, infile))
		return

	long_lines = long_results.split('\n')
	first_line_parts = long_lines[0].split()
	if len(first_line_parts) <= len(base_lines[0].split()):
		options_ok = 0
		test_run.failure("Long listing (-l) did not produce more entries per line", long_results)

	if "Max-sub" not in first_line_parts:
		options_ok = 0
		test_run.failure("Long listing (-l) is missing 'Max-sub' column", long_results)
	if "Ms" not in first_line_parts or "count" not in first_line_parts:
		options_ok = 0
		test_run.failure("Long listing (-l) is missing 'Ms count' column", long_results)

	# only show top 20 lines
	rcode, limit_results = test_run.do_command_result("scripts/kd -n 20 %s" % infile)
	if rcode:
		test_run.failure("Error %s running kd on %s" % (rcode, infile))
		return

	limit_lines = limit_results.split('\n')
	if len(limit_lines) > 22:
		options_ok = 0
		test_run.failure("Limit listing (-n 20) did not limit kd output", limit_results)

	# filter by time (only show functions with total time > 100000 usecs)
	rcode, time_limit_results = test_run.do_command_result("scripts/kd -t 100000 %s" % infile)
	if rcode:
		test_run.failure("Error %s running kd on %s" % (rcode, infile))
		return

	time_limit_lines = time_limit_results.split('\n')
	if len(time_limit_lines) >= len(base_lines):
		options_ok = 0
		test_run.failure("Time limit listing (-t 100000) did not limit kd output", limit_results)

	# sort by local-time
	rcode, local_sort_results = test_run.do_command_result("scripts/kd -s l %s" % infile)
	if rcode:
		test_run.failure("Error %s running kd on %s" % (rcode, infile))
		return

	local_sort_lines = local_sort_results.split('\n')
	
	# look for function names, sorted by local time
	lines = result.split('\n')
	sample_locs = [3,6,10]

	samples = []

	i = 0
	for line in local_sort_lines:
		i += 1
		if i in sample_locs:
			# line should have "function count time average local"
			tuple = line.split()
			# save function, local_time
			samples.append((i, tuple[0], tuple[4]))

	print_log("samples=%s" % samples)

	# check for decreasing times
	if not( int(samples[0][2]) > int(samples[1][2]) and \
		int(samples[1][2]) > int(samples[2][2]) ):
		options_ok = 0
		test_run.failure("function durations not decreasing in Local column")


	# test calltree generation
##############	SKIP for now
#	rcode, calltree_results = test_run.do_command_result("scripts/kd -c -l %s" % infile)
#	if rcode:
#		test_run.failure("Error %s running kd on %s" % (rcode, infile))
#		return
#
#	calltree_lines = calltree_results.split('\n')
#
#	# FIXTHIS - not sure how to test call tree output

	if options_ok:
		test_run.success("KD command line options are OK")


#############################################
# KFT-010 - test timing accuracy
#
def test_010(test_run):
	test_run.set_id("010", "test timing accuracy")
	print_log("Running test %s..." % test_run.id)

	kernel_filename = os.path.join(test_run.build_dir, "vmlinux")

	# remove existing kernel file, if present
	if os.path.isfile(kernel_filename):
		os.unlink(kernel_filename)
		if os.path.isfile(kernel_filename):
			test_run.failure("Could not remove pre-existing kernel image file from build dir.")
			sys.exit(1)

	rcode = test_run.do_ttc("set_config CONFIG_KFT=y", 1)
	rcode = test_run.do_ttc("set_config CONFIG_KFT_STATIC_RUN=y", 1)
	rcode = test_run.do_ttc("set_config CONFIG_ALP_TIMING_TEST=y", 1)
	rcode = test_run.do_ttc('set_config "CONFIG_CMDLINE+=\\" quiet\\""', 1)

	# do make oldconfig to resolve outstanding dependencies
	test_run.do_make_oldconfig()

	# check that config has correct values
	if not test_run.check_config("CONFIG_KFT","y"):
		test_run.failure("Could not set CONFIG_KFT=y in kernel configuration")
		sys.exit(1)

	if not test_run.check_config("CONFIG_ALP_TIMING_TEST","y"):
		test_run.failure("Could not set CONFIG_ALP_TIMING_TEST=y in kernel configuration")
		# don't exit, can still try some tests

	fd = open("kernel/kftstatic.conf","w")
	fd.write(
"""begin
   trigger start entry start_kernel
   trigger stop entry to_userspace
   filter mintime 5000
end""")
	fd.close()

	test_run.set_localversion()

	# build kernel
	rcode = test_run.do_ttc("kbuild")
	if rcode:
		test_run.failure("Could not build kernel")

	# there should be a vmlinux at root of build directory.
	if not os.path.isfile(kernel_filename):
		test_run.failure("No 'vmlinux' found after build")

	# install kernel
	rcode = test_run.do_ttc("kinstall", 1)
	if rcode:
		test_run.failure("Could not install kernel")

	# reset the board - use strongest reboot possible
	test_run.reset_target("reset")
	if not test_run.check_localversion():
		test_run.reset_target("reboot")
		if not test_run.check_localversion():
			test_run.failure("Could not verify kernel running on target board")


	if not test_run.check_localversion():
		test_run.failure("Could not verify kernel running on target board")

	# check if appropriate files are available
	rcode, result = test_run.run_on_target("cat /proc/kft") 
	if rcode:
		test_run.failure("Could not cat /proc/kft")

	rcode, result = test_run.run_on_target("cat /proc/kft_data >/tmp/boot-verify.log") 
	if rcode:
		test_run.failure("Could not cat /proc/kft_data")

	log_ok = collect(test_run, "boot-verify")
	if not log_ok:
		return

	# check for basic boot-verify.log goodness
	test_run.trace_run_id = "0"
	log_ok = check_log_file(test_run, 1000,
		["Execution time filter count"])
	if log_ok:
		test_run.success("Data in log file %s looks OK" % test_run.log_filename)

	test_run.do_command("cp %s/System.map %s" % (test_run.build_dir, test_data_dir))

	# convert to "boot-verify.lst"
	gen_listfile(test_run, "boot-verify")

	infile = test_run.list_filename
	lines = open(infile).readlines()

	in_funcs = 0
	measured_delays = []
	for line in lines:
		if not in_funcs: 
			if line.startswith("-------"):
				in_funcs = 1
			continue
		tuple = line.split()
		func_delay = tuple[1]
		func_addr = tuple[3]
		if func_addr=="test_delay":
			func_delay = int(func_delay)
			measured_delays.append(func_delay)

	test_run.result("measured_delays=%s" % measured_delays)

	# now get dmesg output to compare against requested values
	rcode, result = test_run.run_on_target("dmesg | grep ALP") 
	if rcode:
		test_run.failure("Could not get dmesg results from target")

	dlines = result.split("\n")
	req_delays = []
	for dline in dlines:
		if dline.startswith("ALP timing test middle") or dline.startswith("ALP timing test end"):
			req_delay = dline.split()[4]
			req_delay = int(req_delay)
			if req_delay > 5000:
				req_delays.append(req_delay)

	test_run.result("requested delays (from dmesg) =%s" % req_delays)

	failure = 0
	for i in range(len(measured_delays)):
		measured_delay = measured_delays[i]
		req_delay = req_delays[i]
		delta = abs(req_delay-measured_delay)
		percent = (delta * 100) / req_delay

		test_run.result("Delay delta of %d for requested delay of %d = error of %d%%" %  (delta, req_delay, percent) )

		# allow up to 5% error 
		if (percent > 5):
			failure = 1
			test_run.failure("Delay error of %d is greater than 5% : %d measured for %d requested " % (delta, measured_delay, req_delay))

	if not failure:
		test_run.success("All measured delays were within 5% of requested delays")
		

def test_cleanup(test_run):
	#os.unlink(test_run.tmp_bootlog_filename)
	pass
	
def main():
	global test_run_g

	# get list of valid targets
	(rcode, result) = commands.getstatusoutput('ttc list -q')
	tlist = result.split("\n");
	if rcode:
		print "Error: Problem running 'ttc list'"
		sys.exit(1)

	if '-h' in sys.argv:
		usage()
		sys.exit(1)

	if '-V' in sys.argv:
		print "kft-test.py Ver. %d.%d.%d" % (MAJOR_VERSION, MINOR_VERSION, REVISION)
		sys.exit(1)

	if '-l' in sys.argv:
		print "Available tests:"
		keys = available_tests.keys()
		keys.sort()
		for key in keys:
			desc = available_tests[key]
			print "  %s : %s" % (key, desc)
		sys.exit(1)

	test_list = ['1','2','3','4','5','6','7','8','9','10']
	if '-t' in sys.argv:
		test_list = sys.argv[sys.argv.index('-t')+1]
		sys.argv.remove('-t')
		sys.argv.remove(test_list)
		test_list = test_list.split(',')
		print "Only performing tests: ", test_list

	# Parse the command line
	if len(sys.argv)<2:
		print "Error: missing target to run test on."
		usage()
		print "Available targets are:"
		for t in tlist:
			print "   %s" % t
		print
		sys.exit(1)

	target = sys.argv[1]

	# make sure there are no conflicts - check for existing
	# TC_TARGET environment variable
	if os.environ.has_key("TC_TARGET"):
		print "Error: found TC_TARGET in environment."
		print "Please do not run this test inside an existing ttc target environment."
		print "Aborting test."
		sys.exit(1)

	test_run = ttc_test_utils.test_run_class(test_suite_name, target)
	test_run_g = test_run

	# make directory for test output
	# remove leading ../ from test_data_dir, since we haven't changed
	# to the kernel source dir yet
	test_run.do_command("install -d %s" % test_data_dir[3:], 1)

	test_run.start_log(test_data_dir[3:])

	if target not in tlist:
		test_run.failure("Error: target '%s' not found in ttc configuration" % target)
		print_log("\n###########################################")
		print_log("Results summary:")
		test_run.show_summary_results()
		sys.exit(1)

	# set up test directories
	test_setup(test_run)

	# run remainder of tests
	if '1' in test_list:
		test_001(test_run) # set KFT options, build, boot, check /proc
		test_001a(test_run) # check boot.log data
	if '2' in test_list:
		test_002(test_run) # test addr2sym
	if '3' in test_list:
		test_003(test_run) # test kd (basic usage)
	if '4' in test_list:
		test_004(test_run) # test dynamic trace (entry/exit trigger, mintime filter)
	if '5' in test_list:
		test_005(test_run) # test time trigger
	if '6' in test_list:	# exit trigger alre
		pass	# exit trigger already tested in KFT-004
		#test_006(test_run) # test exit trigger 
	if '7' in test_list:
		test_007(test_run) # test mintime filter
	if '8' in test_list:
		test_008(test_run) # test noints filter
	if '9' in test_list:
		test_009(test_run) # test kd command-line options
	if '10' in test_list:
		test_010(test_run) # test timing accuracy

	test_cleanup(test_run)
	test_run.close_log()

	print_log("\n###########################################")
	print_log("Results summary:")
	test_run.show_summary_results()

if __name__=="__main__":
	main()
