#!/usr/bin/python
#
# size-log-to-wiki.py - a program to convert log results from
# config-sizes-test.py into a wiki table (in MoinMoin format)
#

import os, sys
import re
import commands

default_wiki_path = "oak:/usr/share/moin/alpwiki/data/text/ConfigSizeTestResultsCurrent"

# define some colors:
red1 = "#ff8080"
red2 = "#ff9090"
red3 = "#ffb0b0"
red4 = "#ffd0d0"
green = "#d0ffd0"
blue = "#80d0ff"
blue = "#90e0ff"
yellow = "#ffff90"
lightyellow = "#ffffd0"

title_color = "#80b0c0"
error_color = red1
config_error_color = red2
build_error_color = red3
run_error_color = red4
good_ksize_color = green
good_rsize_color = blue

def usage():
	print """Usage: %s [<options>] <logfile>

Reads the indicated log file and produces a wiki page with a table
of test results.  The page is in MoinMoin format (that is, it uses
MoinMoin wiki markup), and it is written to the file 'wiki_page'
in the current directory.

Options:
-h	Show this usage help
-v	Show the table of results (as well as writing it).
-w <path> Write moin table of results to the indicated location 
	(if '-' is specified, use the default path in the script)
""" % os.path.basename(sys.argv[0])

tiny_options=["AIO",
	"BINFMT_ELF_AOUT",
	"BINFMT_SCRIPT",
	"CC_FUNIT_AT_A_TIME",
	"CONSOLE_TRANSLATIONS",
	"CRC32_TABLES",
	"DOUBLEFAULT",
	"ELF_CORE",
	"ETHTOOL",
	"FILE_LOCKING",
	"FULL_PANIC",
	"IDE_HWIFS",
	"IGMP",
	"INETPEER",
	"LINUXTINY_DO_UNINLINE",
	"MAX_SWAPFILES_SHIFT",
	"MEMPOOL",
	"NET_DEV_MULTICAST",
	"NET_SK_FILTER",
	"NR_LDISCS",
	"PANIC",
	"PCI_QUIRKS",
	"SERIAL_PCI",
	"SYSENTER",
	"XATTR",
]

# some linux-tiny patches affect alread-mainlined options
tiny_options_some=["NET_SMALL",
	"PRINTK_FUNC",
	"PROC_FS",
]
	

class data_item_class:
	def __init__(self, name):
		self.name = name
		self.size = "unavailable"
		self.rsize = "unavailable"
		self.config_error = ""
		self.build_error = ""
		self.run_error = ""
		self.error = ""


class data_class:
	def __init__(self):
		self.data = {}

	def __getitem__(self, key):
		return self.data[key]

	def keys(self):
		return self.data.keys()

	def has_key(self, key):
		return self.data.has_key(key)

	# allows update of an existing data_item value, or creation
	# of a new data_item with the requested attribute value, if none
	# is already in the database
	def set_value(self, key, attr_name, value):
		# validate attr_name here
		# that is, don't allow anything not defined by default
		# comment this out for free-form instances

		#item = data_item_class("foo")
		#if not hasattr(item, attr_name):
		#	raise KeyError, "invalid attribute assignment"

		if self.data.has_key(key):
			setattr(self.data[key], attr_name, value)
		else:
			item = data_item_class(key)
			setattr(item, attr_name, value)
			self.data[key] = item


def write_data_line(fd, format, baseline, item):
	try:
		name, value = item.name.split('=')
	except:
		name = item.name
		value = "-"
		if name=="smallest":
			value="[1]"
	size = item.size
	rsize = item.rsize


	name_attr=""
	value_attr="<:>"
	config_ok = "<:>yes"
	build_ok = "<:>yes"
	runs_ok = "<:>yes"

	# highlight items from Linux-tiny
	if name in tiny_options:
		name_attr ="<%s>" % yellow

	if name in tiny_options_some:
		name_attr ="<%s>" % lightyellow

	# if some weird error, flag the name in red
	if item.error:
		name_attr = "<%s>" % error_color

	if item.config_error:
		config_ok = "<:%s> no" % config_error_color
		build_ok = "<:%s> unavailable" % build_error_color
		runs_ok = "<:%s> unavailable" % run_error_color

	if item.build_error:
		build_ok = "<:%s> no" % build_error_color
		runs_ok = "<:%s> unavailable" % run_error_color

	if item.run_error:
		runs_ok = "<:%s> no" % run_error_color


	# calculate size color
	color = ""
	err = (size.find("unavailable") != -1)
	if err:
		color = build_error_color
	size_attr = "<)%s>" % color

	# calculate delta (and color)
	delta = "unavailable"
	if baseline.size and baseline.size != "unavailable" and not err \
		and size != "unavailable":
		delta = str(int(baseline.size) - int(size))
		if int(delta)>10000:
			color = good_ksize_color

	# make the delta cell right-justified, and add color
	delta_attr = "<)%s>" % color

	# calculate rdelta
	rcolor = ""
	rerr = (rsize.find("unavailable") != -1)
	if rerr:
		rcolor = run_error_color
	rsize_attr = "<)%s>" % rcolor

	rdelta = "unavailable"
	if baseline.rsize and baseline.rsize != "unavailable" and \
		not rerr and rsize != "unavailable":
		rdelta = str(int(rsize) - int(baseline.rsize))
		if int(rdelta)>50000:
			rcolor = good_rsize_color

	# make the rdelta cell right-justified
	rdelta_attr = "<)%s>" % rcolor

	# write the line
	fd.write( format % (name_attr, name,
		value_attr, value,
		config_ok,
		build_ok,
		size_attr,size,
		delta_attr,delta,
		runs_ok,
		rsize_attr, rsize,
		rdelta_attr,rdelta))

def write_table(fd, data):
	format = "||%s%-25s||%s%5s||%s||%s||%s%10s||%s%10s||%s||%s%10s||%s%10s||\n"
	
	line = format % ("""<:rowbgcolor="%s">""" % title_color,"'''Option'''",
		"<:>","'''Value'''",
		"<:>'''Config[[BR]]OK?'''",
		"<:>'''Build[[BR]]OK?'''",
		"<:>","'''Kernel[[BR]]Size'''",
		"<:>","'''Size reduction[[BR]]vs.[[BR]]baseline'''",
		"<:>'''Runs[[BR]]OK?'''",
		"<:>","'''Free memory[[BR]]at runtime'''",
		"<:>","'''Free memory[[BR]]vs.[[BR]]baseline'''")
	fd.write( line )

	# write the line for the baseline kernel at the top

	if data.has_key("baseline"):
		baseline = data["baseline"]
		runs_ok = "<:>yes"	# assume it runs OK
		if baseline.run_error:
			runs_ok = "<:%s> no" % run_error_color

		fd.write( format % ("","baseline",
			"<:>","-",
			"<:>yes",
			"<:>yes",	# assume it builds OK
			"<)>", baseline.size,
			"<:>", "-",
			runs_ok,
			"<)>", baseline.rsize,
			"<:>", "-"))
	else:
		baseline = data_item_class("baseline")
	
	# now write all the option lines

	# get the list of keys, and sort it
	keys = data.keys()
	keys.sort()

	# process each item
	for key in keys:
		if key=="baseline" or key=="smallest":
			continue
		write_data_line(fd, format, baseline, data[key])

	# write the line for the smallest kernel at the bottom
	if data.has_key("smallest"):
		write_data_line(fd, format, baseline, data["smallest"])

def write_wiki_data(wiki_path, data, time, target, host, logfilename):
	outfilename = "wiki_page"
	fd = open(outfilename, "w")

	fd.write("== Test of config sizes ==\n")
	fd.write(" * test time: %s\n" % time)
	fd.write(" * target:    %s\n" % target)
	fd.write(" * host:      %s\n" % host)
	fd.write(" * log file:  %s\n" % os.path.basename(logfilename))

	fd.write("\n=== Results summary table ===\n")

	write_table(fd, data)

	if data.has_key("smallest"):
		fd.write("[1] - See config.smallest\n\n")

	fd.write("""Key:
||<:rowbgcolor="%s">Color||<:>Meaning||
||<:%s>red||error (different shades for different errors)||
||<:%s>green||>10k reduction in kernel size||
||<:%s>blue||>50k increase in runtime memory||\n""" % \
(title_color, config_error_color, good_ksize_color, good_rsize_color) )

	fd.write("""\n=== Warning ===
||<bgcolor="#d09090"> /!\ '' '''Warning''': This file is generated automatically from size-log-to-wiki.py - Any edits you make will be lost.'' ||\n\n""")
	fd.close()
	
	# now copy the data
	if wiki_path:
		cmd = "scp %s %s" % (outfilename, wiki_path)
		(rcode, result) = commands.getstatusoutput(cmd)
		if rcode:
			print "Error copying data to wiki (check permissions and path?)"
			print "result=", result
		print "You can examine the data at:\n %s" % wiki_path
		if wiki_path.startswith("oak:/usr/share/moin/alpwiki/data/text"):
			print "See http://oak.sm.sony.co.jp/alpwiki/%s" % wiki_path[38:]

def main():
	wiki_path = ""

	if '-h' in sys.argv:
		usage()
		sys.exit(1)

	verbose = 0
	if '-v' in sys.argv:
		verbose = 1
		sys.argv.remove("-v")

	if '-w' in sys.argv:
		wiki_path = sys.argv[sys.argv.index("-w")+1]
		sys.argv.remove("-w")
		sys.argv.remove(wiki_path)
		if wiki_path == "-":
			wiki_path = default_wiki_path
		print "Will try to write wiki page to: %s" %  wiki_path

	# check for a remaining arg
	if len(sys.argv)<2:
		print "Error: missing logfile to parse."
		usage()
		sys.exit(1)
	else:
		logfilename = sys.argv[1]

	try:
		fd = open(logfilename)
	except:
		print "Error trying to open %s" % logfilename
		usage()
		sys.exit(1)

	# now, do some parsing
	time = ""
	target = ""
	host = ""
	test = ""
	data = data_class()

	total_size_re = re.compile(".TEST: Size-test-(.*)\] RESULT - kernel total size: ([0-9]*)")
	free_mem_re = re.compile(".TEST: Size-test-(.*)\] RESULT - free memory: ([0-9]*)")
	config_error_re = re.compile(".TEST: Size-test-(.*)\] FAILURE - (Could not set configure option.*)")
	build_error_re = re.compile(".TEST: Size-test-(.*)\] FAILURE - (Could not build kernel.*)")
	run_error_re = re.compile(".TEST: Size-test-(.*)\] FAILURE - .*(not running on board.*)")
	error_re = re.compile(".TEST: Size-test-(.*)\] FAILURE - (.*)")
	for line in fd.readlines(): 
		if line.startswith("time ="):
			time = line[6:].strip()
			print "parsed time=", time
			continue

		if line.startswith("target ="):
			target = line[8:].strip()
			print "parsed target=", target
			continue

		if line.startswith("host ="):
			host = line[6:].strip()
			print "parsed host=", host
			continue

		m = total_size_re.match(line)
		if m:
			print "total size match:", m.groups()
			key = m.groups()[0]
			size = m.groups()[1].strip()
			data.set_value(key, "size", size)
			continue

		m = free_mem_re.match(line)
		if m:
			print "free memory match:", m.groups()
			key = m.groups()[0]
			rsize = m.groups()[1].strip()
			data.set_value(key, "rsize", rsize)
			continue

		m = config_error_re.match(line)
		if m:
			print "config_error match:", m.groups()
			key = m.groups()[0]
			config_error = m.groups()[1].strip()
			data.set_value(key, "config_error", config_error)
			continue

		m = build_error_re.match(line)
		if m:
			print "build_error match:", m.groups()
			key = m.groups()[0]
			build_error = m.groups()[1].strip()
			data.set_value(key, "build_error", build_error)
			continue

		m = run_error_re.match(line)
		if m:
			print "run_error match:", m.groups()
			key = m.groups()[0]
			run_error = m.groups()[1].strip()
			data.set_value(key, "run_error", run_error)
			continue

		m = error_re.match(line)
		if m:
			print "error match:", m.groups()
			key = m.groups()[0]
			error = m.groups()[1].strip()
			data.set_value(key, "error", error)
			
	fd.close()

	if verbose:
		write_table(sys.stdout, data)

	write_wiki_data(wiki_path, data, time, target, host, logfilename)




main()
