#!/usr/bin/python
#
# target-log-tdb-to-wiki.py - a program to convert log results from
# target-test.py into a wiki table (in MoinMoin format)
#

import os, sys
import re
import commands

my_path = os.path.dirname(sys.argv[0])
sys.path.append(my_path)

import tdb

default_wiki_path = "oak:/usr/share/moin/alpwiki/data/text/TargetTestResultsCurrent"

# define some colors:
red1 = "#ff8080"
red2 = "#ff9090"
red3 = "#ffb0b0"
red4 = "#ffd0d0"
green = "#d0ffd0"
blue = "#80d0ff"
blue2 = "#90e0ff"
yellow = "#ffff90"
lightyellow = "#ffffd0"

title_color = "#80b0c0"
error_color = red1
config_error_color = red2
build_error_color = red3
run_error_color = red4
good_build_color = blue
good_boot_color = green

def usage():
	print """Usage: %s [<options>] <logfile>

Reads the indicated log file(s) and produce a wiki page with a table
of test results.  The page is in MoinMoin format (that is, it uses
MoinMoin wiki markup), and it is written to the file 'wiki_page'
in the current directory.

Options:
-h	Show this usage help
-v	Show the table of results (as well as writing it).
-w <path> Write moin table of results to the indicated location 
	(if '-' is specified, use the default path in the script)
""" % os.path.basename(sys.argv[0])

# list has (attr_name, field size, Title)
attr_output_list = [
	("time",25, "Date/Time"),
	("target",7, "Target"),
	("targ_001_list_verify_ok",8, "Target[[BR]]present?"),
	("target_002_info_result",8, "Info"),
	("target_003_get_kernel",8, "Get Kernel"),
	("target_004_get_config",8, "Get config"),
	("target_005_kbuild",8, "Build"),
	("target_006_unique_id",8, "Set/Detect[[BR]]Unique ID"),
	("target_008_boot",8, "Boot"),
	("target_010_copy_to",8, "Copy To" ),
	("target_011_copy_from",8, "Copy From"),
]


def write_data_line(fd, format, record, baseline=None):
	global attr_output_list
	fd.write("||")

	# could use comparison against baseline for test for regression
	# FIXTHIS - ignore baseline for now

	moin_attr = ""
	for field in attr_output_list:
		name = field[0]
		size = field[1]
		try:
			value = record[name]
		except:
			value = "<not available>"
			moin_attr = "<%s>" % error_color

		# handle results that are in standard format
		# i.e. SUCCESS vs. FAILURE
		if value == "FAILURE":
			moin_attr = "<%s>" % error_color

		format = "%s" + "%%%ds" % size + "||"
		fd.write( format % (moin_attr, value))
	fd.write("\n")


def write_table(fd, db):
	fd.write("||")

	# write out title line
	first_field = 1
	for field in attr_output_list:
		if first_field==1:
			moin_attr="""<:rowbgcolor="%s">""" % title_color
			first_field = 0
		else:
			# center titles
			moin_attr="<:>"
		size = field[1]
		title = field[2]

		format = "%s" + "%%%ds" % size + "||"
		fd.write( format % (moin_attr, title))
	fd.write("\n")

	# now write all the option lines

	# get the list of keys, and sort it
	keys = db.keys()
	keys.sort()

	# process each item
	for key in keys:
		write_data_line(fd, format, db[key])

def write_wiki_data(wiki_path, db, tdb_filename):
	outfilename = "wiki_page"
	fd = open(outfilename, "w")

	fd.write("== Test of target control functionality ==\n")
	fd.write(" * tdb file:  %s\n" % os.path.basename(tdb_filename))

	fd.write("\n=== Results summary table ===\n")

	write_table(fd, db)

	fd.write("""Key:
||<:rowbgcolor="%s">Color||<:>Meaning||
||<:%s>red||error (different shades for different errors)||
||<:%s>blue||kernel builds OK||
||<:%s>green||kernel runs OK on target||\n""" % \
(title_color, config_error_color, good_build_color, good_boot_color) )

	fd.write("""\n=== Warning ===
||<bgcolor="#d09090"> /!\ '' '''Warning''': This file is generated automatically from target-log-tdb-to-wiki.py - Any edits you make will be lost.'' ||\n\n""")
	fd.close()
	
	# now copy the data
	if wiki_path:
		cmd = "scp %s %s" % (outfilename, wiki_path)
		(rcode, result) = commands.getstatusoutput(cmd)
		if rcode:
			print "Error copying data to wiki (check permissions and path?)"
			print "result=", result
		print "You can examine the data at:\n %s" % wiki_path
		if wiki_path.startswith("oak:/usr/share/moin/alpwiki/data/text"):
			print "See http://oak.sm.sony.co.jp/alpwiki/%s" % wiki_path[38:]

def main():
	wiki_path = ""

	if '-h' in sys.argv:
		usage()
		sys.exit(1)

	verbose = 0
	if '-v' in sys.argv:
		verbose = 1
		sys.argv.remove("-v")

	if '-w' in sys.argv:
		wiki_path = sys.argv[sys.argv.index("-w")+1]
		sys.argv.remove("-w")
		sys.argv.remove(wiki_path)
		if wiki_path == "-":
			wiki_path = default_wiki_path
		print "Will try to write wiki page to: %s" %  wiki_path

	# check for a remaining arg
	if len(sys.argv)<2:
		print "Error: missing tdb file to read."
		usage()
		sys.exit(1)
	else:
		tdb_filename = sys.argv[1]

	try:
		fd = open(tdb_filename)
	except:
		print "Error trying to open %s" % tdb_filename
		usage()
		sys.exit(1)

	db = tdb.read_tdb_file(tdb_filename, tdb_filename)

	if verbose:
		write_table(sys.stdout, db)

	write_wiki_data(wiki_path, db, tdb_filename)




main()
