#!/usr/bin/python
#
# target-log-to-tdb.py - a program to convert log results from
# target-test.py into a tdb database
#
# extend tdb syntax to support a global record (which functions like
# a record class).  syntax=
# record=.globals
# attr1=value1
#
# all values in .global are inherited by all records
#

debug = 0

fields_spec = """g_time=^time = (.*)
g_target=^target = (.*)
g_host=^host = (.*)
rec_id=^target = (.*)
g_target=^target = (.*)
targ_001_list_verify_ok=\[TEST: TARG-001\] (SUCCESS) - target .* is in 'ttc list'
targ_002_info_result=.TEST: TARG-002\] (SUCCESS|FAILURE) -
targ_002_image_name=.TEST: TARG-002. RESULT - kernel image name is '(.*)'
targ_003_get_kernel=.TEST: TARG-003\] (SUCCESS|FAILURE) -
targ_004_get_config=.TEST: TARG-004\] (SUCCESS|FAILURE) -
targ_005_kbuild=.TEST: TARG-005\] (SUCCESS|FAILURE) -
targ_006_unique_id=.TEST: TARG-006\] (SUCCESS|FAILURE) -
#targ_007_not_completed=.TEST: TARG-007\] (SUCCESS|FAILURE) -
targ_008_boot=.TEST: TARG-008\] (SUCCESS|FAILURE) -
targ_010_copy_to=.TEST: TARG-010\] (SUCCESS|FAILURE) -
targ_011_copy_from=.TEST: TARG-011\] (SUCCESS|FAILURE) -
#targ_016_not_completed=.TEST: TARG-016\] (SUCCESS|FAILURE) -

endpat=^Summary:
"""

# each field spec consists of name=regex
# if the regex includes a group, this is used to extract
# the value for the field.  If there is no group, then the value for a valid
# match is "1"
# if name starts with g_, it is a global, to be applied to be put in a special
# "global" record
#
# special patterns are: 'startpat' which signals the beginning of the
# search space, and 'endpat' which signals the end of the search space
#

import os, sys
import re
import commands


def dprint(msg):
	if debug:
		sys.stderr.write(msg+"\n")
		sys.stderr.flush()

def usage():
	print """Usage: %s [<options>] <logfile>

Reads the indicated log file and produces a file with a database
of test results.  The file is in tdb format, and it is written
to the file 'test-results.tdb' in the current directory.

Options:
-h	Show this usage help
-v	Show the results (as well as writing it).
""" % os.path.basename(sys.argv[0])


class field_spec:
	def __init__(self, name, regex, global_flag):
		self.name = name
		self.regex = regex
		self.rc = re.compile(regex)
		self.gflag = global_flag
	def is_global(self):
		return self.gflag
	def match(self, str):
		self.m = self.rc.match(str)
		return self.m
	def value(self):
		try:
			return self.m.group(1)
		except:
			return "1"

# returns a list of fields_spec instances
def parse_parse_spec(parse_spec):
	field_specs = []
	for line in parse_spec.split('\n'):
		line = line.strip()
		if not line or line.startswith("#"):
			continue
		name, regex = line.split("=",1)
		name = name.strip()
		global_flag = 0
		if name.startswith("g_"):
			global_flag = 1
			name = name[2:]
		instance = field_spec(name, regex, global_flag)
		field_specs.append(instance)

	return field_specs

# parse_log returns a database (which is a map of maps)
def parse_log(fd, fields_spec):
	field_specs = parse_parse_spec(fields_spec)

	# now, do some parsing
	records = {}
	global_rec = {"rec_id":".globals"}
	records[global_rec["rec_id"]] = global_rec
	record = {}
	rec_id = ""

	# find some special patterns
	startpat = endpat = None
	for field_spec in field_specs:
		if field_spec.name=="startpat":
			startpat = field_spec
		if field_spec.name=="endpat":
			endpat = field_spec

	in_search_space = 1
	if startpat:
		in_search_space = 0
	line_no = 0
	for line in fd.readlines(): 
		line_no += 1
		dprint("%d. line=%s" % (line_no, line))
		if not in_search_space:
			if startpat.match(line):
				in_search_space = 1
				# FIXTHIS - this consumes the matching line, maybe it should be parsed?
			continue
		if in_search_space and endpat:
			if endpat.match(line):
				in_search_space = 0
				break
		for field_spec in field_specs:
			dprint("   %s=%s" % (field_spec.name, field_spec.regex))
			if field_spec.match(line):
				dprint("found match")
				dprint("value=%s" % field_spec.value())
				if field_spec.name=="rec_id":
					# found start of new record
					rec_id = field_spec.value()
					record = {"rec_id":rec_id}
					if record:
						records[record["rec_id"]] = record
			
				if not record and not field_spec.is_global():
					print "Error: found match for field %s outside a record definition" % field_spec.name
					print "line=",line
					print "regex=", field_spec.regex
					continue
				if field_spec.is_global():
					global_rec[field_spec.name] = field_spec.value()
					continue
				record[field_spec.name] = field_spec.value()

	return records

def write_db(fd, db, filename):
	fd.write("# test result database from file %s\n" % filename)
	# FIXTHIS - don't output the .globals record.  tdb (as of 2008.06.16)
	# doesn't support it.  For now, output global values in each record.
	try:
		global_rec = db[".globals"]
	except:
		global_rec = None

	keys = db.keys()
	keys.sort()
	keys.remove(".globals")
	
	for rec_id in keys:
		record = db[rec_id]
		fd.write("record=%s\n" % rec_id)
		if global_rec:
			attrs = global_rec.keys()
			attrs.sort()
			for attr in attrs:
				if attr != "rec_id":
					fd.write("%s=%s\n" % (attr, global_rec[attr]))
	
		attrs = record.keys()
		attrs.sort()
		for attr in attrs:
			if attr != "rec_id":
				fd.write("%s=%s\n" % (attr, record[attr]))
		fd.write("\n#########################\n")
	fd.flush()

def main():
	if '-h' in sys.argv:
		usage()
		sys.exit(1)

	verbose = 1 
	if '-v' in sys.argv:
		verbose = 1
		sys.argv.remove("-v")

	# check for a remaining arg
	if len(sys.argv)<2:
		print "Error: missing logfile to parse."
		usage()
		sys.exit(1)
	else:
		logfilename = sys.argv[1]

	try:
		fd = open(logfilename)
	except:
		print "Error trying to open %s" % logfilename
		usage()
		sys.exit(1)

	db = parse_log(fd, fields_spec)
	fd.close()

	fd = open("test-results.tdb", "w")	
	write_db(fd, db, logfilename)
	fd.close()

	if verbose:
		write_db(sys.stdout, db, logfilename)
	

main()
