#!/usr/bin/python
"""
Treemap builder using pylab.

originally based on algorithm straight from http://hcil.cs.umd.edu/trs/91-03/91-03.html
original written by: James Casbon 29/7/2006
See http://www.scipy.org/Cookbook/Matplotlib/TreeMap

Lots of modification by Tim Bird, 4/18/2013
* Use nodes, with attributes holding color, text, etc.
* support different coloring algorithms
* support different layout algorithms
  * I'd really like to support squarify

Squarify algorithm from: "Squarified Treemaps"
Mark Bruls, Kees Huizin and Jark J. van Wijk 
http://www.win.tue.nl/~vanwijk/stm.pdf

todo:
 * add file system scan (as test case)
 * draw from top down instead of bottom up
 * clip text to box
 * add squarified layout method
"""

import pylab
from matplotlib.patches import Rectangle
import matplotlib
import sys
import os

debug = 0

class tree_node:
    def __init__(self, size, color, text=""):
	self.size = size
	self.color = color
	self.text = text
	self.level = 0
	self.children = []
	self.total_size = -1

    def get_color(self):
	return self.color

    def get_text(self):
	size = self.get_size()
	if show_in_k:
		size = size/1024
	return self.text+" - %d" % size

    def get_size(self):
	if self.total_size != -1:
		return self.total_size
	self.total_size = self.size
	for child in self.children:
		self.total_size += child.get_size()
	return self.total_size

    def add_child(self, child):
	child.level = self.level+1
	self.children.append(child)

class dl_node(tree_node):
    def __init__(self, pathname, level=0):
	self.pathname = pathname	# not really needed
	self.level = level
	self.children=[]
	self.total_size = -1
	self.text = os.path.basename(os.path.abspath(pathname))
	try:
		s = self.text.decode()
	except:
		print "found a string with weird chars: %s" % self.text
		try:
			s = unicode(self.text, 'utf-8')
			self.text = s
		except:
			# don't know what's in string, just strip out
			# at this point, it's probably a corrupt filename
			# strip it down to 7-bit ascii
			clean_text = ""
			for i in range(len(self.text)):
				ch = self.text[i]
				if ord(ch) < 128:
					clean_text += ch
				else:
					clean_text += '?'
			self.text = clean_text

	if os.path.isdir(pathname):
		child_list = os.listdir(pathname)
		for child in child_list:
			child_node = dl_node(pathname+os.path.sep+child, level+1)
			self.children.append(child_node)
		# self.size = ???

	self.size = os.lstat(pathname).st_size

class Treemap:
    def __init__(self, tree, color_method, show_text=1):
        """create a tree map from tree, using itermethod(node) to walk tree,
        size_method(node) to get object size and color_method(node) to get its 
        color"""

        self.ax = pylab.subplot(111,aspect='equal')
        pylab.subplots_adjust(left=0, right=1, top=1, bottom=0)
        self.ax.set_xticks([])
        self.ax.set_yticks([])

        self.color_method = color_method
	self.show_text = show_text
	if show_text:
		self.box_yoffset=0.03
	else:
		self.box_yoffset=0.0

	# this layout splits on longest axis and is order-preserving
        self.addnode(tree)

	# FIXTHIS - implement a squarifying layout, as well
        #self.addsquares(tree)

    def addnode(self, node, lower=[0,0], upper=[1,1]):
	# 0 = x axis (lower[0], uppper[0])
	# 1 = y axis (lower[1], uppper[1])

	# draw my box
        color= self.color_method(node)
        self.draw_rectangle(lower, upper, color, node.get_text())

	# now draw my children
	# make the sub-box for children occupy 98% of this box
	# that is, adjust the edges by 2%
	# leave room at top for text
	margin = 0.02
	xwidth = upper[0]-lower[0]
	xwidth *= margin
	lower[0] += xwidth
	upper[0] -= xwidth

	yheight = upper[1]-lower[1]
	yheight *= margin
	lower[1] += yheight
	upper[1] -= yheight

	# if there's room, add some space for the text label
	if upper[1] - self.box_yoffset > lower[1]:
		upper[1] -= self.box_yoffset	# leave room for text label

	# determine direction to divide box
	# do it in pieces along the axis with the longer extent
	xwidth = upper[0]-lower[0]
	yheight = upper[1]-lower[1]
	if xwidth>yheight:
		axis=0
	else:
		axis=1

	# split this sub-box on the indicated axis, proportional
	# to the size of each child
        width = upper[axis] - lower[axis]
        try:
            for child in node.children:
                upper[axis] = lower[axis] + (width * float(child.get_size())) / node.get_size()
                self.addnode(child, list(lower), list(upper))
                lower[axis] = upper[axis]

        except TypeError:
            pass

    def draw_rectangle(self, lower, upper, color, text):
	# don't draw boxes that are too small
	if upper[0]-lower[0]<.005:
		return
	if upper[1]-lower[1]<.005:
		return

        if debug:
		print lower, upper

	# Retangle([x,y], width, height...)
        r = Rectangle( lower, upper[0]-lower[0], upper[1] - lower[1],
                   edgecolor='k', facecolor=color)
        self.ax.add_patch(r)

	show_text = self.show_text
	# check if box is too small for text
	if upper[0]-lower[0]<.02:
		show_text = 0
	if upper[1]-lower[1]<.02:
		show_text = 0

	# put text in upper right corner of box
	if show_text:
		# FIXTHIS - make clip_box work
		cbox = matplotlib.transforms.Bbox([lower, upper])
		cbox = matplotlib.transforms.Bbox.from_bounds(0,0,1,1)
		#print "cbox=", cbox

		tx_xoffset = .01
		tx_yoffset = .01
		self.ax.text(lower[0]+tx_xoffset, upper[1]-tx_yoffset, text,
			va="top", rotation=rotate_text)
		#self.ax.text(lower[0]+tx_xoffset, upper[1]-tx_yoffset, text,
		#	va="top", clip_box=cbox)

def build_test_tree():
	# the old tuple-only representation doesn't support
	# nodes with size greater than their children's size sum
	tree = tree_node(5, 'r', "top")
	node1 = tree_node(2, 'y', "foo")
	node1_1 = tree_node(3, 'y', "f1")
	node1_2 = tree_node(3, 'y', "f2")
	node1_2_1 = tree_node(4, 'g', "floozle")
	node1_2_2 = tree_node(4, 'g', "banchie")
	node1_2_3 = tree_node(1, 'g', "grazz")
	node2 = tree_node(0, 'b', "bar")
	node2_1 = tree_node(1, 'y', "b1")
	node2_2 = tree_node(1, 'y', "b2")
	node3 = tree_node(4, 'b', "konk")

	# build the test tree
	tree.add_child(node1)
	tree.add_child(node2)
	node1.add_child(node1_1)
	node1.add_child(node1_2)
	node1_2.add_child(node1_2_1)
	node1_2.add_child(node1_2_2)
	node1_2.add_child(node1_2_3)
	node2.add_child(node2_1)
	node2.add_child(node2_2)

	return tree

def usage():
	print """Usage: %s [options]
Shows a treemap chart, using matplotlib

-h    show this usage
-l    do a directory listing as a test
-n    don't show text in chart
-r    rotate text slightly, to make it more readable
--debug  show debug output
""" % os.path.basename(sys.argv[0])
	sys.exit(0)

def main():
    global debug
    global rotate_text
    global show_in_k

    do_list = 0
    show_text = 1
    show_in_k = 0
    rotate_text = "horizontal"
    if "-l" in sys.argv:
	do_list = 1
	sys.argv.remove("-l")
    if "-n" in sys.argv:
	show_text = 0
	sys.argv.remove("-n")
    if "-r" in sys.argv:
	rotate_text = -30
	sys.argv.remove("-r")
    if "-k" in sys.argv:
	show_in_k = 1
	sys.argv.remove("-k")
    if "-h" in sys.argv:
	usage()
    if "--debug" in sys.argv:
	debug = 1
	sys.argv.remove("--debug")
	
    def tier_color(thing):
	# I'd like to color it red=top level, orang=2nd level, yello=third level, etc. 
	tier = thing.level
	try:
		color = [ (1,0,0), #red
		(1,.5,0), #orange
		(1,1,0), #yellow
		(.5,1,0), #light green
		(0,1,0), #green
		(0,1,1), #cyan
		(0,1,1), #cyan
		(0,.5,1), #medium blue
		][tier]
	except:
		color = (0,0,1)
	return color

    if do_list:
	tree = dl_node(".")
    else:
	tree = build_test_tree()

    tm = Treemap(tree, tier_color, show_text)
    pylab.show()

if __name__ == '__main__':

    main()
