#!/usr/bin/python
#
# ttc-bisect-test.py - help script for performing 'git bisect' on a target
# 	that is managed by 'ttc'
#
# To Do for ttc-bisect-test.py
#
# BUGS:
#  * failure of telnet_exec (ttc run) is not handled properly:
#
# OUTLINE:
# use target from current environment (TTC_TARGET)
# prepare source for test (apply patches, change config)
# get config
# build kernel
# if could not build, return GIT_NO_INDICATION
# install kernel
# reboot
# verify kernel is installed
# run test
# return 0 for success
# or return 1 for failure
# all other failures return 125 (GIT_NO_INDICATION)

MAJOR_VERSION = 0
MINOR_VERSION = 5 
REVISION      = 0

GIT_OK = 0
GIT_SUCCESS = 0
GIT_FAILURE = 1
GIT_NO_INDICATION = 125

# wait up to 15 minutes for boot
MAX_WAIT_TIME = 60 * 15

import os, sys
import ttc_test_utils

######################################
# Define some globals for this test suite

test_suite_name="BISECT"
src_dir = "test-linux"
test_data_dir = "../test-data"

######################################
# Define some convenience classes and functions

def usage():
	prog_name = os.path.basename(sys.argv[0])
	print """Usage: %s

%s is a program to be used in conjunction with 'git bisect', to
find the commit that caused a particular problem to occur on the target.

This program assumes that TTC_TARGET is set (that is, that 'ttc setenv'
has been set, for the target to be tested).

To use this program, find a version of the kernel that works (is GOOD),
and a version of the kernel that does not work (is BAD), and run this
program in conjunction with git, like so:
 $ ttc <target> setenv
 $ git bisect start <bad-commit> <good-commit>
 $ git bisect run %s

-h	show this usage help
-V	show version information
-s	skip to just performing the test on target
   This is meant only for use during test development.  It skips the
   build, install and reboot steps and just performs the actual runtime test.
""" % (prog_name, prog_name, prog_name)

######################################################

def test_setup(test_run):
	test_run.set_id("001", "setting up for test")
	# test run prep
	print "Doing test preparation for %s tests..." % test_suite_name

	# make test build dir
	build_dir = "../test-build/%s" % test_run.target
	rcode = test_run.do_command("install -d %s" % build_dir, 1)

	# set KBUILD_OUTPUT environment variable to build directory
	os.environ["KBUILD_OUTPUT"]=build_dir

	test_run.build_dir = build_dir

	target = test_run.target

	# sanity check that target is valid
	cmd = "ttc %s info" % target
	(rcode, result) = test_run.do_command_result(cmd)
	if rcode:
		test_run.failure("Could not get info for target '%s'" % target, result)
		return GIT_NO_INDICATION
	
	test_run.success("Retrieved info for target '%s'" % target, result)

	test_run.set_id("002", "verify we're in the kernel source tree")

	# look for some 'normal' files at the root of the kernel source
	if not os.path.isfile('MAINTAINERS') or not os.path.isfile('Kbuild'):
		test_run.failure("Missing key files in kernel source directory after 'get_kernel'")
		return GIT_NO_INDICATION
	
	# read Makefile, and get kernel version
	makelines = open("Makefile").readlines()
	version = 0
	minor = 0
	revision = 0
	extra = ""
	for line in makelines:
		if not version and line.startswith('VERSION'):
			version = line.split('=')[1].strip()
		if not minor and line.startswith('PATCHLEVEL'):
			minor = line.split('=')[1].strip()
		if not revision and line.startswith('SUBLEVEL'):
			revision = line.split('=')[1].strip()
		if not extra and line.startswith('EXTRAVERSION'):
			extra = line.split('=')[1].strip()
	ver_str = "%s.%s.%s%s" % (version, minor, revision, extra)
	test_run.result_out("Kernel version is %s" % ver_str)
	test_run.success("Detected kernel version OK")


	test_run.set_id("003", "get default kernel configuration")
	# first, remove old configuration, if present
	config_filename = os.path.join(test_run.build_dir, ".config")
	if os.path.isfile(config_filename):
		os.unlink(config_filename)
		if os.path.isfile(config_filename):
			test_run.failure("Could not remove pre-existing kernel config file from build dir.")
			return GIT_NO_INDICATION

	# get default configuration
	rcode = test_run.do_ttc("get_config", 1)

	# verify that the configuration was placed in the build directory
	if not os.path.isfile(config_filename):
		test_run.failure("Could not get default kernel configuration for target")
		return GIT_NO_INDICATION
	else:
		test_run.success("Got default kernel configuration for target")


	# remove existing kernel file, if present
	kernel_filename = os.path.join(test_run.build_dir, "vmlinux")
	if os.path.isfile(kernel_filename):
		os.unlink(kernel_filename)
		if os.path.isfile(kernel_filename):
			test_run.failure("Could not remove pre-existing kernel image file from build dir.")
			return GIT_NO_INDICATION

	return GIT_OK

######################################################
	
# build the kernel
def do_build(test_run):
	test_run.set_id("004", "build the kernel")

	# set the localversion to something unique
	test_run.set_localversion()
	rcode = test_run.do_ttc("kbuild")
	if rcode:
		test_run.failure("Could not build kernel")
		return GIT_NO_INDICATION

	# test to make sure localversion was built into local kernel image
	id_str = test_run.kernel_id
	rcode = test_run.do_command("grep %s $KBUILD_OUTPUT/vmlinux" % id_str)
	if rcode:
		test_run.failure("Could not build kernel with unique identifier '%s'" % id_str)
		return GIT_NO_INDICATION

	return GIT_OK

######################################################

def do_install_and_reboot(test_run, max_wait_time=60):
	test_run.set_id("004", "install and reboot")

	# install kernel
	rcode = test_run.do_ttc("kinstall", 1)
	if rcode:
		test_run.failure("Could not install kernel")
		return GIT_NO_INDICATION

	# reset the board - use strongest reboot possible
	test_run.reset_target("reboot")

	test_run.wait_for_target_to_boot(max_wait_time)

	# detect that the kernel version running on the target is 
	# what we just built
	if test_run.check_localversion():
		test_run.success("Kernel %s installed and booted on board" % test_run.kernel_id)
	else:
		test_run.failure("Could not verify kernel running on target board")
		return GIT_NO_INDICATION

	return GIT_OK

######################################################

def do_test(test_run):
	# this test checks whether the rsize of the NFS-mounted root filesystem
	# is equal to 4096 or is greater
	# rsize <= 4096 = SUCCESS
	# rsize > 4096 = FAILURE
	
	# check for result of test run on target
	rcode, result = test_run.run_on_target("mount")
	if rcode:
		test_run.failure("Error collecting results from 'mount'", result)
		return GIT_NO_INDICATION

	result_list = result.split("\n")
	# find rsize in /dev/root mount line
	rsize = -1
	for line in result_list:
		if line.find("rsize=") != -1:
			(first, rest) = line.split("rsize=",1)
			rsize = int(rest.split(",")[0])
	if rsize==-1:
		test_run.failure("Error parsing results from 'mount'", result)
		return GIT_NO_INDICATION

	if rsize > 4096:
		test_run.failure("rsize for /dev/root (over NFS) is %d" % rsize)
		return GIT_FAILURE
	else:
		test_run.success("rsize for /dev/root (over NFS) is %d" % rsize)
		return GIT_SUCCESS
		

######################################################

def main():
	# Parse the command line
	if '-h' in sys.argv:
		usage()
		return GIT_NO_INDICATION

	if '-V' in sys.argv:
		prog_name = os.path.basename(sys.argv[0])
		print "%s Ver. %d.%d.%d" % (prog_name, MAJOR_VERSION,
			MINOR_VERSION, REVISION)
		return GIT_NO_INDICATION

	skip_stuff = 0
	if '-s' in sys.argv:
		skip_stuff = 1

	# get target from TTC_TARGET environment variable
	try:
		target = os.environ["TTC_TARGET"]
	except:
		print "Error: Could not find TTC_TARGET in environment."
		print "Please run this test inside an existing ttc target environment."
		print "Aborting test."
		return GIT_NO_INDICATION

	test_run = ttc_test_utils.test_run_class(test_suite_name, target)

	# make directory for test output
	# remove leading ../ from test_data_dir, since we haven't changed
	# to the kernel source dir yet
	test_run.do_command("install -d %s" % test_data_dir[3:], 1)

	test_run.start_log(test_data_dir[3:])

	# prepare for test
	result = test_setup(test_run)
	if result != GIT_OK:
		test_run.close_log()
		return result

	# try compilation
	if not skip_stuff:
		result = do_build(test_run)
		if result != GIT_OK:
			test_run.close_log()
			return result

	# try install and reboot (with kernel version verification)
	if not skip_stuff:
		max_wait_time = MAX_WAIT_TIME
		result = do_install_and_reboot(test_run, max_wait_time)
		if result != GIT_OK:
			test_run.close_log()
			return result

	# do actual test
	result = do_test(test_run)

	test_run.close_log()
	return result

if __name__=="__main__":
	result = main()
	sys.exit(result)
