#!/usr/bin/python
#
# wrclient.py - a webrelay client
#
# This program can be used to view and set the state of
# relays in a webrelay quad device.
#
# Written by: Tim Bird (March, 2007)   tim dot bird (at) am dot sony dot com
#
# This program is is placed in the public domain.
#

import sys, os
import urllib
import re

# specify server to use by default
#default_server = "192.168.1.2"
default_server = "webrelay"

#### user should be of form: "user_name:user_password@"
user = ""

# list of proxies, if any, to use for request
proxies={}

# customize these names to match your webrelay setup
# tuples are in format: ("<name>", "<relay number>")
relay_names = [
	("osk4", "1"),
	("tx49", "2"),
	("relay3", "3"),
	("relay4", "4"),
	("1", "1"),
	("2", "2"),
	("3", "3"),
	("4", "4"),
]

# you can customize these state names to match your webrelay setup
# tuples are in format: ("<name", "<state value>")
state_names = [
	("on", "0"),
	("off", "1"),
	("normal", "0"),
	("switched", "1"),
	("0","0"),
	("1","1"),
]

# customize these status names to match your webrelay setup
# each row is a relay, and the columns are for states 0 and 1, respectively
status_messages = [("dummy", "dummy"),
	("osk4 power on", "osk4 power off"), 		# 1
	("tx49 power on", "tx49 power off"),		# 2
	("relay 3 normal", "relay 3 switched"),		# 3
	("relay 4 normal", "relay 4 switched"),		# 4
]

verbose = 0

# turn a list of tuples into a comma-separated string
# routine assumes the first element of the tuple is the identifier
def get_list_str(list):
	list_str = "  "
	for item in list[:-1]:
		list_str += '"'+item[0]+'", '
	# add last item
	item = list[-1]
	list_str += 'or "'+item[0]+'"'
	return list_str

def usage():
	rnames_str = get_list_str(relay_names)
	snames_str = get_list_str(state_names)

	print """Usage: %s [options] [<relay> <state>]

View or set the state of relays on a webrelay quad device.

If no relay and state are specified, then just print
the current status of all the relays.  If a relay and state
are specified, change the state of the specified relay, then
show the status of all relays.

-h, --help   Show this usage help
-v	     Show verbose output
-s <server>  Specify the server (by name or address) to use

<relay>	     Name or number of relay to set.  Should be one of:
	       %s
<state>	     State to set for relay.  Should be one of:
	       %s
""" % (os.path.basename(sys.argv[0]), rnames_str, snames_str)
	sys.exit(0)

def vprint(msg):
	global verbose
	if verbose:
		print msg

# returns (server, relay, state)
def parse_command_line(args):
	global verbose

	server = default_server
	relay = None
	state = None

	# get rid of program name
	args = args[1:]

	# check for options
	# remove them as they are found to keep the arg count right
	if "-h" in args or "--help" in args:
		usage()
	if "-v" in args: 
		verbose = 1
		args.remove('-v')
	if "-s" in args: 
		server = args[args.index('-s')+1]
		args.remove('-s')
		args.remove(server)
	
	# see if we should parse <relay> and <state>
	if len(args) == 0:
		# OK, return without parsing relay and state
		return (server, relay, state)

	if len(args) != 2:
		print "Error: incorrect arguments"
		usage()

	relay_arg = args[0]
	for alias in relay_names:
		if alias[0] == relay_arg:
			relay = alias[1]
	if not relay:
		print "Error: invalid relay '%s'" % relay_arg
		usage()

	state_arg = args[1]
	for alias in state_names:
		if alias[0] == state_arg:
			state = alias[1]
	if not state:
		print "Error: invalid state '%s' for relay %s" % (state_arg, relay_arg)
		usage()

	return (server, relay, state)

def show_status(status):
	# should do real XML parse here, but this should work
	for relay in range(1,5):
		regex = "<relay%dstate>(.*?)<" % relay
		m = re.search(regex, status)
		state = -1
		if not m:
			print "Error: missing status for relay %d in response from server!!" % relay
		else:
			state = m.groups()[0]
			if state!= "1" and state != "0":
				print "Error: unexpected state value of '%s' for relay '%d' in response" % (state, relay)
			else:
				state = int(state)

		# look up the message for this relay and state
		if state != -1:
			message = status_messages[relay][state]
			print "Relay %d, Status: %s (%d)" %  (relay, message, state)
		else:
			print "Relay %d, Status: <UNKNOWN>"
		
		

def main():
	global verbose

	(server, relay, state) = parse_command_line(sys.argv)

	vprint("Getting data from %s" % server)

	if not relay:
		# just get the state
		url = "http://%s%s/state.xml" % (user, server)
	else:
		# send relay and state as part of the HTTP request
		url = "http://%s%s/state.xml?relay%sState=%s" % \
			(user, server, relay, state)

	f = urllib.urlopen(url, proxies=proxies)

	response = f.read()

	vprint("Response from server is:\n----\n%s----" % response)

	show_status(response)


# this can be used as an imported module or a standalone program
if __name__=="__main__":
	main()
